<?php

namespace Teknisa\FIN\Modules\ContaPagar\Factory;

use Teknisa\Libs\Util\DateUtil;
use Doctrine\ORM\EntityManager;
use Teknisa\Model\Fiscal\Entities\Notafiscal;
use Teknisa\Model\Financeiro\Entities\Boletpagar;
use Teknisa\Model\Financeiro\Entities\Contapagar;
use Teknisa\Model\Financeiro\Entities\Ajctpagar;
use Teknisa\Model\Financeiro\Entities\Tipoajpagar;

/**
 * Class ProrrogacaoTituloFactory
 * @package Teknisa\FIN\Modules\ContaPagar\Factory
 */
class ProrrogacaoTituloFactory {

    protected $entityManager;

    /**
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }


    /**
     * Busca Boleto a Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolpag
     * @return BoletPagar
     */
    public function buscaBoletPagar($nrorg, $cdfilial, $nrlanbolpag){
        /** @var $params */
        $params = array();
        $params['nrorg']       = $nrorg;
        $params['cdfilial']    = $cdfilial;
        $params['nrlanbolpag'] = $nrlanbolpag;

        /** @var Boletpagar $boletpagar */
        $boletpagar = $this->entityManager->getRepository(Boletpagar::class)->findOneBy($params);

        return $boletpagar;
    }

    /**
     * Busca Contas a Pagar
     * @param $nrorg
     * @param $cdfilibpag
     * @param $nrlanbolpag
     * @return Contapagar
     */
    public function buscaContaspagarAssoc($nrorg, $cdfilibpag, $nrlanbolpag){
        /** @var $params */
        $params = array();
        $params['nrorg']       = $nrorg;
        $params['cdfilibpag']  = $cdfilibpag;
        $params['nrlanbolpag'] = $nrlanbolpag;

        /** @var Contapagar $contapagar */
        $contapagar = $this->entityManager->getRepository(Contapagar::class)->findBy($params);

        return $contapagar;
    }

    /**
     * Busca Ajustes Conta Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @return Ajctpagar
     */
    public function buscaAjsctpagar($nrorg, $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag){
        /** @var $params */
        $params = array();
        $params['nrorg']        = $nrorg;
        $params['cdfilial']     = $cdfilial;
        $params['nrlanctonf']   = $nrlanctonf;
        $params['nrseqvenpag']  = $nrseqvenpag;
        $params['nrparcelapag'] = $nrparcelapag;

        /** @var Ajctpagar $ajctpagar */
        $ajctpagar = $this->entityManager->getRepository(Ajctpagar::class)->findBy($params);

        return $ajctpagar;
    }

    /**
     * Busca Tipo de Ajuste Pagar
     * @param $nrorg
     * @param $cdajuctpag
     * @return Tipoajpagar
     */
    public function buscaTipoajpagar($nrorg, $cdajuctpag){
        /** @var $params */
        $params = array();
        $params['nrorg']      = $nrorg;
        $params['cdajuctpag'] = $cdajuctpag;

        /** @var Tipoajpagar $tipoajpagar */
        $tipoajpagar = $this->entityManager->getRepository(Tipoajpagar::class)->findOneBy($params);

        return $tipoajpagar;
    }

    /**
     * Busca Nota Fiscal
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanctonf
     * @return Notafiscal
     */
    public function buscaNotafiscal($nrorg, $cdfilial, $nrlanctonf){
        /** @var $params */
        $params = array();
        $params['nrorg']      = $nrorg;
        $params['cdfilial']   = $cdfilial;
        $params['nrlanctonf'] = $nrlanctonf;

        /** @var Notafiscal $notafiscal */
        $notafiscal = $this->entityManager->getRepository(Notafiscal::class)->findOneBy($params);

        return $notafiscal;
    }
}