<?php
namespace Teknisa\FIN\Modules\ContaReceber\Services\Emissao\Banco;

use Teknisa\Libs\Util\DateUtil;
use Teknisa\FIN\Modules\ContaReceber\Services\Emissao\GeraEmissaoBoletoService;

/**
 * Class GeraEmissaoBoletoSicoobService
 * @package Teknisa\FIN\Modules\ContaReceber\Services\Emissao\Banco
 */
class GeraEmissaoBoletoSicoobService extends GeraEmissaoBoletoService
{

    /**
     * Tamanho do operador da linha digitavel de nosso numero.
     */
    const TAMANHO_NOSSO_NUMERO                   = 8;
    const MODULO_DIGITO_VERIFICADOR_NOSSONUM     = 11;
    const MODULO_DIGITO_VERIFICADOR_LINHADIGIT   = 10;
    const FATOR_VENCIMENTO                       = '07/10/1997';


    public function getNossoNumeroComDigito()
    {
        $codigoReduzido = $this->getCodigoReduzido();
        $cdcontador      = self::PREFIXO_CDCONTADOR.$codigoReduzido;
        $nossoNumero     = \Geral\Util\NovoCodigo::geraCodigo (
            $this->boleto->getNrorg(), $cdcontador, 1, self::TAMANHO_NOSSO_NUMERO
        );
        
        return $nossoNumero;
    }


    /** Função que monta a Linha Digitável do boleto de acordo com o Layout do Banco Bradesco.
     * @param $nossoNumero
     * @return string
     */
    public function getLinhaDigitavel($nossoNumero)
    {
        
        $campoLivre =   substr($this->boleto->getCdcartcobbol(), -1).
                        substr(str_pad($this->getCdAgencia(), 4, 0, STR_PAD_LEFT), 0, 4).
                        '01'.
                        substr(str_pad($this->getCdcedenteban(), 7, 0, STR_PAD_LEFT), -7).
                        substr(str_pad($nossoNumero, 8, 0, STR_PAD_LEFT), -8).
                        substr(str_pad($this->boleto->getNrparcbolrec(), 3, 0, STR_PAD_LEFT), 0, 3);
                        
        $grupo1 = $this->getIdentificadorBanco() . '9' . substr($campoLivre, 0, 5) . self::getVerificadorGruposLinhaDigitavel($this->getIdentificadorBanco() . '9' . substr($campoLivre, 0, 5));
        $grupo2 = substr($campoLivre, 5, 10) . self::getVerificadorGruposLinhaDigitavel(substr($campoLivre, 5, 10));
        $grupo3 = substr($campoLivre, 15, 10) . self::getVerificadorGruposLinhaDigitavel(substr($campoLivre, 15, 10));
        $grupo4 = $this->getDigitoCodigoBarras();
        $grupo5 = $this->getFatorVencimento() . $this->getValorNominal();

        return self::concatLinhaDigitavel($grupo1, $grupo2, $grupo3, $grupo4, $grupo5);

    }

    public function getCodigoDeBarras($nossoNumero)
    {
        $campoLivre =   substr($this->boleto->getCdcartcobbol(), -1).
                        substr(str_pad($this->getCdAgencia(), 4, 0, STR_PAD_LEFT), 0, 4).
                        '01'.
                        substr(str_pad($this->getCdcedenteban(), 7, 0, STR_PAD_LEFT), -7).
                        substr(str_pad($nossoNumero, 8, 0, STR_PAD_LEFT), -8).
                        substr(str_pad($this->boleto->getNrparcbolrec(), 3, 0, STR_PAD_LEFT), 0, 3);
        
        //Result := Copy(stLinhaAux, 01, 04) + // Banco(3) + Código da Moeda(1)
        $codigoDeBarras  = $this->getIdentificadorBanco();
        $codigoDeBarras .= self::MOEDA_REAL;
        // Dígito Verificador do código de barras(1)
        $codigoDeBarras .= "_SUBSTITUICAO_DV_";
        // Fator de Vencimento
        $codigoDeBarras .= $this->getFatorVencimento();
        // Valor Nominal do Título no bloqueto
        $codigoDeBarras .= $this->getValorNominal();
        
        // Campo Livre Cobrança Com Registro
        // Código da carteira
        $codigoDeBarras .= substr($this->boleto->getCdcartcobbol(), -1);
        // Código da agência
        $codigoDeBarras .= str_pad(substr($this->getCdAgencia(), 0, 4), 4, "0", STR_PAD_LEFT);
        // 12ª a 20ª posição do Campo Livre
        $codigoDeBarras .= substr($campoLivre, 5, 10);
        // 22ª a 31ª posição do Campo Livre
        $codigoDeBarras .= substr($campoLivre, 15, 10);

        $digito         = $this->getDigitoVerificador(str_replace("_SUBSTITUICAO_DV_", "", $codigoDeBarras));
        $codigoDeBarras = str_replace("_SUBSTITUICAO_DV_", $digito, $codigoDeBarras);
        
        return $codigoDeBarras;
    }

    private function getValorNominal()
    {
        $valorBoleto = (string)$this->getValorBoleto();
        $valorBoleto = str_replace(',','.',$valorBoleto);
        $valorBoleto = number_format($valorBoleto, 2, '.', '');
        $arrayExclusao = array('.', ',');
        $valorBoleto = str_replace($arrayExclusao, "", $valorBoleto);
        $valorBoleto = str_pad($valorBoleto, 10, "0", STR_PAD_LEFT);
        return $valorBoleto;
    }


    private function getCodigoCedente()
    {
        if ($this->getCdcedenteban() == false) {
            $codigoCedente = '0000';
        } else {
            $codigoCedente = substr($this->getCdcedenteban(), 0, 4);
        }

        return $codigoCedente;
    }


    private function getIdentificadorBanco()
    {
        $identificadorBanco = $this->getCdbanccnab();
        return $identificadorBanco;
    }

    private function getFatorVencimento()
    {
        $datavenc            = DateUtil::truncateData($this->getDataVencimento());
        $dataFatorVencimento = DateUtil::truncateData(DateUtil::getDataDeString(self::FATOR_VENCIMENTO));
        $fatorDeVencimento   = DateUtil::getQtdeDiasEntre($dataFatorVencimento, $datavenc);
        $fatorDeVencimento   = str_pad($fatorDeVencimento, 4, "0", STR_PAD_LEFT);
        return $fatorDeVencimento;
    }

    private function getVerificadorGruposLinhaDigitavel($linhaDigitavel)
    {
        $linhaDigitavel = strrev($linhaDigitavel);
        $peso = 2;
        $total = 0;
		$multiplo = 0;
        for($i = 0; $i<=strlen($linhaDigitavel);$i++){
            $result = (int) substr($linhaDigitavel,$i,1)* $peso;
            if ($result > 9){
                $total += (int)substr((string)$result, 0, 1) + (int)substr((string)$result, 1,1);
            }else{
                $total += ((int) substr($linhaDigitavel,$i,1)* $peso);
            }
            $peso = $this->incrementaValorPeso($peso, 1, 2 );

       }

        $digito = $total % 10;
	    while($multiplo < $digito){
			$multiplo = $multiplo + 10;
		}

        return $multiplo - $digito;
    }

    public function getRelatorioBoleto($nrorg, $cdreducont){

        /** @var $contcorr \Geral\Entities\Contcorr */
        $contcorr = \Geral\Entities\Contcorr::findOneByNrorgAndCdreducont($nrorg,$cdreducont);
        $contcorr->getcdbanco();

        /** @var $dsarqbolcomp Banco*/
        $dsarqbolcomp = \Geral\Entities\Banco::findOneByCdbanco($contcorr->getcdbanco());
        $string = $dsarqbolcomp->getDsarqbolcomp();
        $tam = strlen($string);
        $tam_b = strrpos($string,'\\');
        $ext = -($tam - strrpos($string,'.'));
        if($tam_b === false){
            return NULL;
        }
        else
            return ($string_p = substr($string, ++$tam_b, $ext));
    }

	private function concatLinhaDigitavel($grupo1,$grupo2,$grupo3,$grupo4,$grupo5){

       	$linhadigitavel = substr($grupo1,0,5).'.'.substr($grupo1, 5).' ';

       	$linhadigitavel .= substr($grupo2,0, 5).'.'.substr($grupo2, 5).' ';

       	$linhadigitavel .= substr($grupo3,0, 5).'.'.substr($grupo3, 5).' ';

       	$linhadigitavel .= $grupo4.' ';

       	$linhadigitavel .= $grupo5;

       	return $linhadigitavel;
    }

    public function chamaRelatorio(){
        $nomeArquivo = $this->getCdfilial().$this->getNrlanbolrec();
        $arquivo = APP_PATH . DIRECTORY_SEPARATOR . "vendor" . DIRECTORY_SEPARATOR . "setforms" . DIRECTORY_SEPARATOR . "libraries" . DIRECTORY_SEPARATOR . "lib". DIRECTORY_SEPARATOR . "dompdf" . DIRECTORY_SEPARATOR . $nomeArquivo . '.html';

        ob_start();

        $params = $this->handleBoleto();

        $html = ob_get_clean();

        $htmlFile = fopen($arquivo, "w");
        fwrite($htmlFile, $html);
        fclose($htmlFile);

        require_once(APP_PATH . DIRECTORY_SEPARATOR . "vendor" . DIRECTORY_SEPARATOR . "setforms" . DIRECTORY_SEPARATOR . "libraries" . DIRECTORY_SEPARATOR . "lib". DIRECTORY_SEPARATOR . "dompdf" . DIRECTORY_SEPARATOR . "dompdf_config.inc.php");
        $dompdf = new \DOMPDF();
        $dompdf->load_html_file($arquivo);
        $dompdf->render();
        unlink($arquivo);
        $outfile = APP_PATH . DIRECTORY_SEPARATOR . "public" . DIRECTORY_SEPARATOR . "reports" . DIRECTORY_SEPARATOR . $nomeArquivo . '.pdf';
        file_put_contents($outfile, $dompdf->output( array("compress" => 0) ));

        return $params;
    }

    public function handleBoleto(){

        // DADOS DO BOLETO PARA O SEU CLIENTE
        $dias_de_prazo_para_pagamento = 5;
        $taxa_boleto = 0;
        $data_venc = $this->getDataVencimento()->format('d/m/Y'); // Prazo de X dias OU informe data: "13/04/2006";
        $valor_cobrado = $this->getValorBoleto(); // Valor - REGRA: Sem pontos na milhar e tanto faz com "." ou "," ou com 1 ou 2 ou sem casa decimal
        $valor_cobrado = str_replace(",", ".",$valor_cobrado);
        $valor_boleto=number_format($valor_cobrado+$taxa_boleto, 2, ',', '');

        $dadosboleto["nosso_numero"] = $this->getNossoNumeroComDigito();
        $dadosboleto["numero_documento"] = $this->getNrnotafisc();	// Num do pedido ou do documento
        $dadosboleto["data_vencimento"] = $data_venc; // Data de Vencimento do Boleto - REGRA: Formato DD/MM/AAAA
        $dadosboleto["data_documento"] = date("d/m/Y"); // Data de emiss�o do Boleto
        $dadosboleto["data_processamento"] = date("d/m/Y"); // Data de processamento do boleto (opcional)
        $dadosboleto["valor_boleto"] = $valor_boleto; 	// Valor do Boleto - REGRA: Com v�rgula e sempre com duas casas depois da virgula

        // DADOS DO SEU CLIENTE
        $dadosboleto["sacado"] = $this->getCliente();
        $dadosboleto["endereco1"] = $this->getDsendeclie();
        $dadosboleto["endereco2"] = $this->getMunicipioCliente() . " - " . $this->getEstadoCliente() . " - " . " CEP: " . $this->getCepCliente();

        // INSTRU��ES PARA O CAIXA
        $dadosboleto["instrucoes1"] = $this->getInstrucao1();
        $dadosboleto["instrucoes2"] = $this->getInstrucao2();

        // DADOS OPCIONAIS DE ACORDO COM O BANCO OU CLIENTE
        $dadosboleto["quantidade"] = " ";
        $dadosboleto["valor_unitario"] = " ";
        $dadosboleto["aceite"] = "N";
        $dadosboleto["especie"] = "R$";
        $dadosboleto["especie_doc"] = "DM";


        // ---------------------- DADOS FIXOS DE CONFIGURA��O DO SEU BOLETO --------------- //


        // DADOS DA SUA CONTA - BANCO DO SICOOB
        if(strlen($this->getCdAgencia()) > 4)
            $dadosboleto["agencia"] = substr($this->getCdAgencia(), 0, 4); // Num da agencia, sem digito
        else
            $dadosboleto["agencia"] = $this->getCdAgencia();

        if(strlen($this->getCdcontcorr()) > 7)
            $dadosboleto["conta"] = substr($this->getCdcontcorr(), 0, 7);
        else
            $dadosboleto["conta"] = $this->getCdcontcorr(); // Num da conta, sem digito

        // DADOS PERSONALIZADOS - BANCO DO SICOOB
        $dadosboleto["convenio"] = $this->getCdcedenteban();  // Num do conv�nio - REGRA: 6 ou 7 ou 8 d�gitos
        $dadosboleto["contrato"] = ""; // Num do seu contrato

        if(strlen($this->getCarteira()) > 2)
            $dadosboleto["carteira"] = substr($this->getCarteira(), 1, 2);
        else if(strlen($this->getCarteira()) < 2)
            $dadosboleto["carteira"] = str_pad($this->getCarteira(), 2, 0, STR_PAD_LEFT);
        else
            $dadosboleto["carteira"] = $this->getCarteira();
        $dadosboleto["variacao_carteira"] = "";  // Varia��o da Carteira, com tra�o (opcional)

        // TIPO DO BOLETO
        $dadosboleto["formatacao_convenio"] = strlen($this->getCdcedenteban()); // REGRA: 8 p/ Conv�nio c/ 8 d�gitos, 7 p/ Conv�nio c/ 7 d�gitos, ou 6 se Conv�nio c/ 6 d�gitos
        $dadosboleto["formatacao_nosso_numero"] = "2"; // REGRA: Usado apenas p/ Conv�nio c/ 6 d�gitos: informe 1 se for NossoN�mero de at� 5 d�gitos ou 2 para op��o de at� 17 d�gitos

        // SEUS DADOS
        $dadosboleto["cedente"] = $this->getNmepresa();
        $dadosboleto["cpf_cnpj"] = $this->getNrinsjurempr();

        include_once("BoletoPhp/include/funcoes_bb.php");
        include("BoletoPhp/include/variaveis_bb.php");
        include("BoletoPhp/include/layout_bb.php");

        return array(
            "CDBARRAREC" => $dadosboleto['codigo_barras'],
            "DSLINHABARREC" => $dadosboleto["linha_digitavel"],
            "CDNOSNUMREC" => $dadosboleto["nosso_numero"]
        );
    }

}
?>